# Code snippet that they have to add to (spelling bee?)
import random
def generate_answers(letters: list, center: str) -> dict:
    """ Gather English words that are valid answers to the spelling bee letters 

    Parameters:
        english (list): List of English words
        letters (list): List of non-center characters in spelling bee
        center (str): Center character in spelling bee

    Returns:
        validWords (dict): Dictionary of valid answers to spelling bee 
                            (all values are 1)
    """
    with open('data/english_words.txt', 'r') as f:
        # Read all words by splitting on new line and remove trailing empyt
        # string
        all_words = f.read().split('\n')[:-1]
        
    validWords = {}
    for word in all_words:
        if inVocab(letters, center, word):
            validWords[word.upper()] = 1
    return validWords
    
def print_honeycomb(letters: list, center: str) -> None:
    """ Prints the honeycomb pattern with the letters

    Parameters:
        letters (list): List of non-center characters in spelling bee
        center (str):aCenter character in spelling bee
    Returns:
        None
    """
    print(f'            _____')
    print(f'           /     \ ')
    print(f'          /       \ ')
    print(f'    ,----(    {letters[0]}    )----.')
    print(f'   /      \       /      \ ')
    print(f'  /        \_____/        \ ')
    print(f'  \   {letters[1]}    /     \    {letters[2]}   / ')
    print(f'   \      /       \      / ')
    print(f"    )----(    {center}    )----( ")
    print(f"   /      \       /      \ ")
    print(f"  /        \_____/        \ ")
    print(f"  \   {letters[3]}    /     \    {letters[4]}   /")
    print(f"   \      /       \      / ")
    print(f"    `----(    {letters[5]}    )----' ")
    print(f"          \       / ")
    print(f"           \_____/ ")

def shuffle(letters: list) -> None:
    """ Shuffles the letters 

    Parameters:
        letters (list): List of non-center characters in spelling bee
    Returns:
        None
    """
    random.shuffle(letters)

def inVocab(letters: list, center: str, word: str) -> bool:
    """ Checks that a word is valid guess. Valid guesses are words which 
    contain the center letter, does not include letters not in letters or center, 
    and is at least 4 letters long. 

    Parameters:
        letters (list): List of non-center characters in spelling bee
        center (str): Center character in spelling bee
        word (str): Word user guessed
    Returns:
        isValid (bool): Whether the word is valid
    """
    onlyBees = True
    hasCenter = False
    for char in word.upper():
        if char not in letters+[center]:
            onlyBees = False
        if char == center:
            hasCenter = True
    isValid = onlyBees and hasCenter
    return isValid

def turn(letters: list, center: str, guesses: list) -> str:
    """ Executes a turn of guessing a word in spelling bee,
    where a turn means the user inputed a valid guess

    Parameters:
        letters (list): List of non-center characters in spelling bee
        center (str): Center character in spelling bee
        guesses (list): Prior guess
    Returns:
        word (str): User's guess
    """
    print_honeycomb(letters, center)
    Valid = False
    word = ''
    while not Valid:
        word = input("Enter a guess or 'S' if you want to shuffle "\
                "the board and print it again: ").upper()
        # Shuffle and print board
        if word == 'S':
            shuffle(letters)
            print()
            print_honeycomb(letters, center)
        elif len(word) < 4:
            print('Words must have at least 4 characters')
        elif not inVocab(letters, center, word):
            print(f'Word must contain only {letters+[center]} '\
                  f'and must have {center}')
        # Guess must be unique
        elif word in guesses:
            print(f"You've already guessed {word}")
        else:
            Valid = True
    return word

def isPangram(word:str, allLetters: list) -> bool:
    """Returns whether a word is a pangram (word containing all letters).
    Parameters:
        word (str): User guess
        allLetters (list): List of all spelling bee characters (including center)
    Returns:
        pangram (bool): If word is pangram
    """
    hasLetter = [0]*len(allLetters)
    for i in range(len(allLetters)):
        if allLetters[i] in word:
            hasLetter[i] = 1
    pangram = False
    if len(hasLetter) == sum(hasLetter):
        pangram = True
    return pangram

def getScore(word: str, possibleWords: dict, allLetters: list) -> int:
    """ Get the score for a valid guess.
    Parameters:
        word (str): User guess
        possibleWords (dict): Dictionary of valid answers to spelling bee
                            (all values are 1)
        allLetters (list): List of all spelling bee characters (including center)
    Returns:
        score (int): Score in spelling bee
    """
    score = 0
    if word in possibleWords:
        if len(word) == 4:
            score = 1
        else:
            score = len(word)
        if isPangram(word, allLetters):
            score += 7
    return score

def main() -> None:
    letters = ["B","D","E","O","V","Y"]
    center = 'R'
    keepGuessing = True
    guesses = []
    numWords = 0
    score = 0
    answers = generate_answers(letters, center)
    while keepGuessing:
        word = turn(letters, center, guesses)
        numWords += 1
        guesses.append(word)
        score += getScore(word, answers, letters+[center])
        print(f"You have guessed {numWords} words for a score of {score}")
        more = input('Keep guessing (y/n)? ')
        if more.lower() == 'n':
            keepGuessing = False
            
    print(f'Thanks for playing!')
    print(f'You guessed {numWords} words for a final score of {score}')

main()
